/*-
 * Copyright (C)2017..2018 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#) Copyright (C)2017..2018 @BABOLO http://www.babolo.ru/"
#ident "@(#) $Id: func.c,v 1.15 2018/07/25 16:34:11 babolo Exp $"

#define BLIN_COMPAT      3
#define Bpars_COMPAT     3
#define MIFE_COMPAT      4
#define RECOBE_COMPAT    4
#define PGOBLIN_COMPAT   2
#define PGOBLIN_INTERNAL 1
#define PGOBLIN_GOSHORT  5
#define PGOBLIN_GOCMD    6

#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sysexits.h>
#include <strings.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include <babolo/recobe.h>
#include <multilar.h>
#include <mife.h>
#include "pgoblin.h"

const u_char *
/************************************************************************
 **                                                                    **/
winpoint(mife_descriptor *md, off_t offset, ssize_t trd) {            /**
 **                                                                    **
 ************************************************************************/
    const u_char    *p = NULL;
    ssize_t          rdr;

    errno = 0;
    if  (trd > (rdr = mife_read(md, trd, offset))) {
        if  (!!mife_ctlsize(md)) {
            ifBLIN_QW0("mife_read");
            if  (rdr >= 0) errno = EFTYPE;
        }
    } else if (!(p = mife_pointer(md))) {
        ifBLIN_QW0("mife_pointer");
    }
    return(p);
}

int
/************************************************************************
 ************************************************************************
 **                                                                    **/
pgoblin_breg(pgoblin_command cmd, pgoblin_nr rn) {                    /**
 **                                                                    **
 ************************************************************************
 **      cmd   rn.     **
 **    cmd  rn  ,   < 0 **
 ************************************************************************
 ************************************************************************/
    int j;

    j = -1;
    if  (pgoblin.a[cmd].flags & pgoblin.p[rn].sym) {
        /*  a[cmd].flags    [rn] */
        j = (pgoblin.p[rn].bin < 0)
            /*      */
          ? (pgoblin.a[cmd].flags >> ~pgoblin.p[rn].bin) & pgoblin.p[rn].mask
            /*      */
          : pgoblin.p[rn].bin
        ;
        if  (!(pgoblin.a[cmd].ch[j] & pgoblin.p[rn].val) && !!pgoblin.a[cmd].ch[j]) {
            /*                            *
             *     ,  -     . *
             *    0 j ,   j  ,      *
             *        j                    *
             */
            j = -2;
    }   }
    return(j);
}

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_cmdexe(int cmd, pgoblin_exenv *exenv, pgoblin_nr *rn) {     /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
    char    null = 0xFF;
    u_char  buff[10];
    char    buf[4];
    int     binreg;
    int     symreg;
    ssize_t strl;
    ssize_t wrt;
    int     ex;

    ifBLIN_QX3( "+%s %c%c%c%c%c%c %08X"
              , pgoblin.a[cmd].str
              , pgoblin_regn[rn[0]]
              , pgoblin_regn[rn[1]]
              , pgoblin_regn[rn[2]]
              , pgoblin_regn[rn[3]]
              , pgoblin_regn[rn[4]]
              , pgoblin_regn[rn[5]]
              , pgoblin.a[cmd].flags
              );
    if  (0 <= pgoblin_options->xtrace) {
        if  (!!flock(pgoblin_options->xtrace, LOCK_EX)) {
            ifBLIN_QW0("xtrace lock");
        } else {
            buf[0] = (cmd >> 8) | 0x0E0;
            buf[1] = cmd & 0x0FF;
            errno = 0;
            if  (2 != (wrt = mife_writ(pgoblin_options->xtrace, buf, 2))) {
                ifBLIN_QW0("mife_writ %d <> 2", wrt);
            }
            bzero(buf, 4);
            for (symreg = 0; symreg < pgoblin.symparam; symreg++) {
                if  (0 <= (binreg = pgoblin_breg(cmd, symreg))) {
                    buf[binreg] = rn[symreg];
            }   }
            errno = 0;
            if  (4 != (wrt = mife_writ(pgoblin_options->xtrace, buf, 4))) {
                ifBLIN_QW0("mife_writ %d <> 2", wrt);
            }
            for (binreg = 0; binreg < pgoblin.binparam; binreg++) {
                if  (PGOBLIN_STYLE & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_JOB & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_CONNECT & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_PQRESULT & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_MIFEDES & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_TEXTPARM & pgoblin.a[cmd].ch[binreg]) {
                    if  (!exenv->options->io[buf[binreg] & 0x0FF].text) {
                        errno = 0;
                        if  (1 != (wrt = mife_writ(pgoblin_options->xtrace, &null, 1))) {
                            ifBLIN_QW0("mife_writ %d <> 1", wrt);
                        }
                        break;
                    } else if (  !!(exenv->options->io[buf[binreg] & 0x0FF].flags & PGOBLIN_BINPARM)
                              || !!exenv->options->io[buf[binreg] & 0x0FF].length
                              ) {
                        strl = exenv->options->io[buf[binreg] & 0x0FF].length;
                    } else {
                        strl = strlen(exenv->options->io[buf[binreg] & 0x0FF].text);
                    }
                    if  (strl > 0x000000007FFFFFFFLL) {
                        ifBLIN_QX0("string too big %"BLIN_X, strl);
                    } else {
                        int l = recobe_cstore(buff, strl);

                        errno = 0;
                        if  (l != (wrt = mife_writ(pgoblin_options->xtrace, buff, l))) {
                            ifBLIN_QW0("mife_writ %d <> %d", wrt, l);
                        }
                        if  (strl != (wrt = mife_writ( pgoblin_options->xtrace
                                                     , exenv->options->io[buf[binreg] & 0x0FF].text
                                                     , strl
                            )        )               ) {
                            ifBLIN_QW0("mife_writ %"BLIN_D" <> %"BLIN_D, wrt, strl);
                }   }   }
                if  (PGOBLIN_PATHPARM & pgoblin.a[cmd].ch[binreg]) {
                    if  (!exenv->options->io[buf[binreg] & 0x0FF].path) {
                        errno = 0;
                        if  (1 != (wrt = write(pgoblin_options->xtrace, &null, 1))) {
                            ifBLIN_QW0("write %d <> 1", wrt);
                        }
                        break;
                    } else {
                        strl = strlen(exenv->options->io[buf[binreg] & 0x0FF].path);
                    }
                    if  (strl > 0x000000007FFFFFFFLL) {
                        ifBLIN_QX0("string too big %"BLIN_X, strl);
                    } else {
                        int l = recobe_cstore(buff, strl);

                        errno = 0;
                        if  (l != (wrt = mife_writ(pgoblin_options->xtrace, buff, l))) {
                            ifBLIN_QW0("mife_writ %d <> %d", wrt, l);
                        }
                        if  (strl != (wrt = mife_writ( pgoblin_options->xtrace
                                                     , exenv->options->io[buf[binreg] & 0x0FF].path
                                                     , strl
                            )        )               ) {
                            ifBLIN_QW0("mife_writ %"BLIN_D" <> %"BLIN_D, wrt, strl);
        }   }   }   }   }
        if  (!!flock(pgoblin_options->xtrace, LOCK_UN)) {
            ifBLIN_QW0("xtrace unlock");
    }   }
    ex = (*(pgoblin.a[cmd].car))(exenv, rn);
    ifBLIN_QX3( "- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

void *
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_getr(pgoblin_main *options, pgoblin_nr rn, int reg) {       /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
#   define blin_internal_flags (options->flags & BLIN_MASK)
    switch(reg) {
    case PGO_CSTY: return(&(options->style[rn]));
    case PGO_CJOB: return(&(options->job[rn]));
    case PGO_CCON: return(&(options->conn[rn]));
    default:       return(&(options->io[rn]));
#   undef blin_internal_flags
}   }

void
/*****************************************************************************
 *****************************************************************************
 **                                                                         **/
pgoblin_go( void *x           /*  (f > 0)   (f < 0)     */
/**/      , int f             /*    pgoblin_syntax.ch[]  *-1 */
/**/      , const char *file  /*                            */
/**/      , const char *func  /*                                   */
/**/      , int line          /*                        */
/**/      ) {                                                              /**
 **                                                                         **
 *****************************************************************************
 *****************************************************************************/
#   define blin_internal_flags (pgoblin_options->flags & BLIN_MASK)
    if  (0 <= pgoblin_options->xtrace) {
        u_char    buf[PGOBLIN_GOSHORT];
        u_int64_t d;
        u_int64_t p;
        u_int64_t q;
        int       i;

        recobe_store(&buf[2], 3, getpid());
        if  (0 < f) {
            switch(f) {
            case PGOBLIN_STYLE   :
            case PGOBLIN_JOB     :
            case PGOBLIN_CONNECT :
            case PGOBLIN_OUTSET  :
            case PGOBLIN_PQRESULT:
            case PGOBLIN_MIFEDES :
            case PGOBLIN_TEXTPARM:
            case PGOBLIN_PATHPARM:
                buf[0] = f;
                buf[1] = (u_int64_t)x;
                break;
            default              :
                buf[0] = 0;
                buf[1] = f;
            }
        } else {
            switch(-f) {
            case PGOBLIN_STYLE   :
                p = (u_int64_t)&pgoblin_options->style[0];
                q = (u_int64_t)&pgoblin_options->style[1];
                break;
            case PGOBLIN_JOB     :
                p = (u_int64_t)&pgoblin_options->job[0];
                q = (u_int64_t)&pgoblin_options->job[1];
                break;
            case PGOBLIN_CONNECT :
                p = (u_int64_t)&pgoblin_options->conn[0];
                q = (u_int64_t)&pgoblin_options->conn[1];
                break;
            case PGOBLIN_OUTSET  :
            case PGOBLIN_PQRESULT:
            case PGOBLIN_MIFEDES :
            case PGOBLIN_TEXTPARM:
            case PGOBLIN_PATHPARM:
                p = (u_int64_t)&pgoblin_options->io[0];
                q = (u_int64_t)&pgoblin_options->io[1];
                break;
            default              :
                buf[0] = 0;
                buf[1] = -f;
                goto out;
            }
            d = ((u_int64_t)x - p) / (q - p);
            if  (((d * (q - p)) != ((u_int64_t)x - p)) || !!(d & ~0x3FUL)) {
                ifBLIN_QX1( "@%s:%s:%d: "
                            "Lost syn %"BLIN_X" i%"BLIN_X" c%"BLIN_X" j%"BLIN_X" s%"BLIN_X" f%02X"
                          , file
                          , func
                          , line
                          , BLIN_I(x)
                          , BLIN_I(&pgoblin_options->io[0])
                          , BLIN_I(&pgoblin_options->conn[0])
                          , BLIN_I(&pgoblin_options->job[0])
                          , BLIN_I(&pgoblin_options->style[0])
                          , -f
                          );
                d = 0x3F;
            }
            buf[0] = -f;
            buf[1] = d;
        }
out:
        errno = 0;
        if  (!!flock(pgoblin_options->xtrace, LOCK_EX)) {
            ifBLIN_QW0("xtrace lock");
        } else {
            errno = 0;
            if  (PGOBLIN_GOSHORT != (i = mife_writ(pgoblin_options->xtrace, buf, PGOBLIN_GOSHORT))) {
                ifBLIN_QW0("mife_writ %d <> %d", i, PGOBLIN_GOSHORT);
            }
            if  (!!flock(pgoblin_options->xtrace, LOCK_UN)) {
                ifBLIN_QW0("xtrace unlock");
    }   }   }
#   undef blin_internal_flags
}

static char P[] = "Xftmqocjs";

static int
/*****************************************************************************
 **                                                                         **/
txtout( pgoblin_main *options                                              /**/
/**/  , mife_descriptor *md                                                /**/
/**/  , off_t *offset                                                      /**/
/**/  , int type                                                           /**/
/**/  , int binreg                                                         /**/
/**/  , int symreg                                                         /**/
/**/  , int rn                                                             /**/
/**/  ) {                                                                  /**
 **                                                                         **
 *****************************************************************************/
#   define blin_internal_flags (options->flags & BLIN_MASK)
    int              ex = EX_OK;
    ssize_t          rdr;
    ssize_t          trd;
    const u_char    *p;

    if  (!(p = winpoint(md, *offset, 1))) {
        ifBLIN_QW0("trace %c", P[type]);
        ex = (errno == EFTYPE) ? EX_DATAERR : EX_IOERR;
        goto out;
    }
    if  (*p == 0x0FF) {
        ++*offset;
        goto out;
    }
    if  (!(p = winpoint(md, *offset, recobe_clen(*p)))) {
        ifBLIN_QW0("trace %c", P[type]);
        ex = (errno == EFTYPE) ? EX_DATAERR : EX_IOERR;
        goto out;
    }
    rdr = recobe_clen(*p);
    trd = recobe_cload(p, &rdr);
    if  (recobe_clen(*p) != rdr) {
        ifBLIN_QX0("recobe_cload <> recobe_clen");
        ex = EX_SOFTWARE;
        errno = EDOOFUS;
        goto out;
    }
    *offset += rdr;
    if  (!(p = winpoint(md, *offset, trd))) {
        ifBLIN_QW0("trace %c", P[type]);
        ex = (errno == EFTYPE) ? EX_DATAERR : EX_IOERR;
        goto out;
    }
    if  (options->flags & PGOBLIN_T_COMAND) {
        printf( "        %d %d [%c]P%c=%.*s~\n"
              , binreg
              , symreg
              , pgoblin_regn[rn]
              , P[type]
              , (int)trd
              , p
              );
    }
    *offset += trd;
out:;
    return(ex);
}

int
/*****************************************************************************
 *****************************************************************************
 **                                                                         **/
pgoblin_traceat( pgoblin_main *options                                     /**/
/**/           , mife_descriptor *(*popin)(pgoblin_main *options)          /**/
/**/           , const char *Tflag                                         /**/
/**/           ) {                                                         /**
 **                                                                         **
 *****************************************************************************
 *****************************************************************************/
#   define blin_internal_flags (options->flags & BLIN_MASK)
    char             t[pgoblin.binparam];
    char             s[pgoblin.symparam];
    int              ex = EX_OK;
    int              seqnum;
    off_t            offset;
    int              binreg;
    int              symreg;
    u_int16_t        cmd;
    mife_descriptor *md;
    const u_char    *p;

    if  (!(md = popin(options))) {
        ifBLIN_QW0("popin mife_init");
        ex = EX_IOERR;
        goto out;
    }
    if  (!Tflag) {
        ifBLIN_QX0("no trace task");
        ex = EX_USAGE;
        errno = EINVAL;
        goto out;
    }
    while(' ' < *Tflag) {
        switch(*(Tflag++)) {
        case 'R': options->flags |= PGOBLIN_T_REGO;   break;
        case 'C': options->flags |= PGOBLIN_T_COMAND; break;
    }   }
    for (offset = 0, seqnum = 0; ; ++seqnum) {
        if  (!(p = winpoint(md, offset, PGOBLIN_GOSHORT))) {
            if  (!mife_ctlsize(md)) break;
            ifBLIN_QW0("trace file");
            ex = (errno == EFTYPE) ? EX_DATAERR : EX_IOERR;
            goto out;
        }
        switch(*p & 0x0FF) {
        case PGOBLIN_STYLE   : t[0] = P[8]; break;
        case PGOBLIN_JOB     : t[0] = P[7]; break;
        case PGOBLIN_CONNECT : t[0] = P[6]; break;
        case PGOBLIN_OUTSET  : t[0] = P[5]; break;
        case PGOBLIN_PQRESULT: t[0] = P[4]; break;
        case PGOBLIN_MIFEDES : t[0] = P[3]; break;
        case PGOBLIN_TEXTPARM: t[0] = P[2]; break;
        case PGOBLIN_PATHPARM: t[0] = P[1]; break;
        case 0               : t[0] = P[0]; break;
        default              : t[0] = 0;
        }
        if  (!!t[0]) {
            ++p;
            if  (options->flags & PGOBLIN_T_REGO) {
                printf( "R%6d P%c %c @%u\n"
                      , seqnum
                      , t[0]
                      , pgoblin_regn[*p & 0x03F]
                      , (unsigned)recobe_load(p + 1, 3)
                      );
            }
            offset += PGOBLIN_GOSHORT;
            continue;
        }
        if  ((*p & 0x0E0) != 0x0E0) {
            ifBLIN_QX0("illegal trace file 0x%02X @ 0x%012"BLIN_O"X", *p, offset);
            ex = EX_DATAERR;
            errno = EILSEQ;
            goto out;
        }
        if  (!(p = winpoint(md, offset, PGOBLIN_GOCMD))) {
            ifBLIN_QW0("trace file");
            ex = (errno == EFTYPE) ? EX_DATAERR : EX_IOERR;
            goto out;
        }
        cmd = (*(p++) << 8) & 0x0100;
        cmd |= *(p++);
        bzero(t, pgoblin.binparam);
        bzero(s, pgoblin.symparam);
        for (symreg = 0; symreg < pgoblin.symparam; symreg++) {
            if  (0 <= (binreg = pgoblin_breg(cmd, symreg))) {
                s[symreg] = t[binreg] = p[binreg];
        }   }
        offset += pgoblin.binparam + 2;
        if  (options->flags & PGOBLIN_T_COMAND) {
            printf( "C%6d #%s %c%c%c%c%c%c %08X\n"
                  , seqnum
                  , pgoblin.a[cmd].str
                  , pgoblin_regn[s[0] & 0x03F]
                  , pgoblin_regn[s[1] & 0x03F]
                  , pgoblin_regn[s[2] & 0x03F]
                  , pgoblin_regn[s[3] & 0x03F]
                  , pgoblin_regn[s[4] & 0x03F]
                  , pgoblin_regn[s[5] & 0x03F]
                  , pgoblin.a[cmd].flags
                  );
        }
        for (symreg = 0; symreg < pgoblin.symparam; symreg++) {
            if  (0 <= (binreg = pgoblin_breg(cmd, symreg))) {
                if  (PGOBLIN_STYLE & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_JOB & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_CONNECT & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_OUTSET & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_PQRESULT & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_MIFEDES & pgoblin.a[cmd].ch[binreg]) {
                }
                if  (PGOBLIN_TEXTPARM & pgoblin.a[cmd].ch[binreg]) {
                    ex = txtout(options, md, &offset, 2, binreg, symreg, t[binreg] & 0x03F);
                }
                if  (PGOBLIN_PATHPARM & pgoblin.a[cmd].ch[binreg]) {
                    ex = txtout(options, md, &offset, 1, binreg, symreg, t[binreg] & 0x03F);
    }   }   }   }
out:;
    return(ex);
#   undef blin_internal_flags
}
